package com.xhly.manageapp.utils

import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.ServiceConnection
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.os.Bundle
import android.os.IBinder
import android.util.Log
import cn.com.microtrust.firewall.afw.AFWUtils
import cn.com.microtrust.firewall.aidl.IAFWService
import com.xhly.corelib.utils.AppUtils


object TB23FCAfwUtils {
    var TAG = "AFW_TEST"
    val PACKAGE_NAME = "cn.com.microtrust.firewall"
    val ACTION = "cn.com.microtrust.firewall.IAFWService"

    private var iafwService: IAFWService? = null
    private val mServiceConnection: ServiceConnection = object : ServiceConnection {
        override fun onServiceDisconnected(name: ComponentName) {
            iafwService = null
            Log.e(TAG, "Service Disconnected")
        }

        override fun onServiceConnected(name: ComponentName, service: IBinder) {
            Log.d(TAG, "onServiceConnected")
            iafwService = IAFWService.Stub.asInterface(service)
            getFireWallStatus()
        }
    }
    private fun getFireWallStatus() {
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.isEnable()
                Log.i(TAG, "获取防火墙状态"+afwRes.operationResult);
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

     fun startBindService(context: Context) {
        Log.d(TAG, "bindAFWService")
        val bundle = Bundle()
        val intent = Intent()
        intent.setAction(ACTION)
        intent.setPackage(PACKAGE_NAME)
        intent.putExtras(bundle)
        context.bindService(intent, mServiceConnection, Context.BIND_AUTO_CREATE)
    }

    /**
     * 添加黑白名单 0为黑名单，1为白名单
     */
     fun jurisdiction(type: Int=0, appUid: Int) {
       try {
           if (type == 0) {
               //黑名单
               if (iafwService != null) {
                   val afwRes = iafwService!!.addAppBlackRule(appUid.toString() + "")
                   Log.i(TAG, "addAppBlackRule result=" + afwRes.operationResult)
               }
           } else if (type == 1) {
               //白名单
               if (iafwService != null) {
                   val afwRes = iafwService!!.addAppWhiteRule(appUid.toString() + "")
                   Log.i(TAG, "addAppWhiteRule result=" + afwRes.operationResult)
               }
           }
       }catch (e:Exception){

       }
    }

    //添加app白名单 2025.3.18
    fun addAppWhiteList(appList:List<String>,context: Context){
        try {
            if (iafwService != null) {
                appList.forEach {app->
                    AFWUtils.addAppWhiteRule(context, iafwService,app)
                }
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }
    /**
     * 使用新方法获得app白名单 2025.3.18
     */
    fun getNetAppWhiteList(context: Context):ArrayList<String>{
        val appList= arrayListOf<String>()
        try {
            if (iafwService != null) {
                val appWhiteRules = AFWUtils.getAppWhiteRules(context, iafwService)
                appList.addAll(appWhiteRules)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
        return appList
    }

    fun getNetAppBlockList():String{
        val sb=StringBuffer()
        sb.append("黑名单应用")
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.blackRules
                val data = afwRes.data
                for (i in data.indices) {
                    Log.i(TAG, "blackRules=" + data[i])
                    sb.append(data[i])
                }
            } else {
                Log.i(TAG, "iafwService=null")
                sb.append("iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
            sb.append("有问题$e")
        }
        return sb.toString()
    }


    /**
     * 添加网址白名单
     */
    fun addUrLWhiteList(urlList:List<String>){
        try {
            if (iafwService != null) {
                urlList.forEach {url->
                    val afwRes = iafwService!!.addWhiteRule(url)
                    Log.i(TAG, "urlsetEnable=" + afwRes.operationResult+"|||"+url)
                }
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 获得网址白名单
     */
    fun getUrlWhiteList():ArrayList<String>{
        val urlList= arrayListOf<String>()
        try {
            if (iafwService != null) {
                val whiteRules = AFWUtils.getWhiteRules(iafwService)
                urlList.addAll(whiteRules)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
        return urlList
    }



    /**
     * 添加网址黑名单
     */
    fun addUrLBlackList(urlList:List<String>){
        try {
            if (iafwService != null) {
               urlList.forEach {url->
                   val afwRes = iafwService!!.addBlackRule(url)
                   Log.i(TAG, "setEnable=" + afwRes.operationResult)
               }
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 清除APP网络规则
     */
    fun clearAppRules(){
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.clearAppRules()
                Log.i(TAG, "清除APP规则setEnable=" + afwRes.operationResult)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 清除IP和网址规则
     */
    fun clearIpHostRules(){
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.clearIpHostRules()
                Log.i(TAG, "clearIpHostRules=" + afwRes.operationResult)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 清除所有规则
     */
    fun clearRules(){
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.clearRules()
                Log.i(TAG, "clearRules=" + afwRes.operationResult)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 打开防火墙开关
     */
    fun setEnable(enable: Boolean) {
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.setEnable(enable)
                addUrLBlackList(arrayListOf("*"))
                Log.i(TAG, "开启网络setEnable=" + afwRes.operationResult)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    /**
     * 固化规则，将规则固化到系统，下次启动应用生效
     */
    fun writeToFile(){
        try {
            if (iafwService != null) {
                val afwRes = iafwService!!.writeToFile()
                Log.i(TAG, "WriteToFile=" + afwRes.operationResult)
            } else {
                Log.i(TAG, "iafwService=null")
            }
        } catch (e: java.lang.Exception) {
            e.printStackTrace()
            Log.e(TAG, e.toString())
        }
    }

    fun getUUid(context: Context,pkgName:String): String {
        return try {
            val packageManager: PackageManager = context.packageManager
            val packageInfo: PackageInfo = packageManager.getPackageInfo(pkgName, 0)
            val uid = packageInfo.applicationInfo.uid
            uid.toString()?:"0"
        } catch (e: java.lang.Exception) {
            "0"
        }
    }
}